/*****************************************************************************/

/*
 *      osremote.h -- REALmagic Celeste Application
 *
 *      Copyright (C) 1999-2000 Sigma Designs
 *                   written by Emmanuel Michon <emmanuel_michon@sdesigns.com>
 *                          and Nicolas Vignal <nicolas_vignal@sdesigns.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <termio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/time.h>
#include <pthread.h>
#include <getopt.h>

// macros
#define interface struct
#define CONST_VTBL
#define __RPC_FAR
#define BEGIN_INTERFACE
#define END_INTERFACE

#ifdef __cplusplus
#define EXTERN_C    extern "C"
#else
#define EXTERN_C    extern
#endif

#define _tprintf printf
#define _stprintf sprintf

#define UNUSED_ARG 0xdeadbeef

#define THREAD_SELF_ID UNUSED_ARG

#define OSD_MANAGER_THREAD_PRIORITY UNUSED_ARG
#define TIMER_THREAD_PRIORITY UNUSED_ARG
#define MAIN_THREAD_PRIORITY UNUSED_ARG
#define PORT_READING_THREAD_PRIORITY UNUSED_ARG

#define S_OK 0
#define ERROR 1

// notice: remote control serial port is chosen by environment REMOTECONTROLPORT
#define REM_PORT NULL 
#define KBD_PORT "/dev/console"

#define TEXT(quote) quote

// int types
typedef int PORTHANDLE,HRESULT;

// 8bit types
typedef char *PCHAR,*LPCTSTR;
typedef unsigned char *PBYTE;

// 16bit types

// 32bit types
typedef long LONG;
typedef unsigned long ULONG;

// pointer types
typedef void *LPVOID;

// function pointer type
typedef void *(*start_routine_type)(void *);
typedef start_routine_type FUNCPTR;

extern int command_line_switch_YUV;
extern int command_line_switch_NTSC;
extern int command_line_switch_PAL;

// MULTITHREADING FUNCTIONS

/* IMPORTANT NOTE ABOUT HOW LINUX HANDLES MULTITHREADING 

   Usually multithreading under Linux is handled by the pthread library.

   pthread library authors do not recommend to mix multithreading
   and signals; but people appreciate correct handling of most used 
   signals (Control-C, KILL).

   To solve this problem, libEM8400 maintains a table of spawned threads 
   and manages them properly. To handle threads use the following functions:
 */

// fmp os exposed types and functions:
typedef void * THREADHANDLE,*MESSAGE_QUEUE_OBJECT;

THREADHANDLE OSCreateThread(char *name,unsigned long dwStackSize,
			    FUNCPTR lpStartAddress,int lpParameter);
void OSCancelThread(THREADHANDLE hThread);

#define MSGOTHERKIND 4012
#define REASONABLE_UMASK 0644
#define MAXMSGLENGTH 320
MESSAGE_QUEUE_OBJECT OSCreatemessagequeue(key_t key, int msgflg);
unsigned long OSDestroyMessageQueue(MESSAGE_QUEUE_OBJECT MsgQ);
void OSPostInmessagequeue(MESSAGE_QUEUE_OBJECT pm,char *msg,int length,long type);
void OSBlockingReceiveFrommessagequeue(MESSAGE_QUEUE_OBJECT pm,char *msg,int length,long type);
// fmp os exposed types and functions (over)

#define _DEFINE_FMP_TYPES_
