/*****************************************************************************/
/*  From XMMS - Cross-platform multimedia player
 *
 *  addfilesbrowser.c - playlist's 'addfiles' browser
 *
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *                           modified by Pascal Cannenterre <pascal_cannenterre@sdesigns.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*****************************************************************************/
#include "define.h"
#include "addfilesbrowser.h"
#include "playlistcb.h"

static GtkWidget *filebrowser = NULL;

void gtk_filebrowser_destroy(GtkWidget *widget, gpointer data)
{
  gtk_widget_destroy(filebrowser);
  filebrowser = NULL;
}

void create_add_files_browser(gboolean clear_pl_on_ok)
{
  if (filebrowser != NULL && filebrowser->window != NULL)
    {
      gdk_window_show(filebrowser->window);
      gdk_window_raise(filebrowser->window);
    }
  else
    { 
      GtkWidget *bbox, *add_selected, *add_all, *label;
      gboolean *ptr;

      filebrowser = gtk_file_selection_new(_("Load file(s)"));
		
      gtk_clist_set_selection_mode(GTK_CLIST(GTK_FILE_SELECTION(filebrowser)->file_list),
				   GTK_SELECTION_EXTENDED);
      gtk_signal_connect(GTK_OBJECT(filebrowser), "destroy",
			 GTK_SIGNAL_FUNC(gtk_filebrowser_destroy), NULL);
      /* gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filebrowser)->selection_entry), */
      /* 			 "changed", GTK_SIGNAL_FUNC(filebrowser_changed), filebrowser); */
      gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filebrowser)->ok_button),
			 "clicked", GTK_SIGNAL_FUNC(filebrowser_ok), filebrowser);
      gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filebrowser)->cancel_button),
				"clicked", GTK_SIGNAL_FUNC(gtk_filebrowser_destroy),
				NULL);

      if (Info.sFilename)
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(filebrowser),
					Info.sFilename);
      bbox = gtk_hbutton_box_new();
      gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
      gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 0);
      gtk_box_pack_end(GTK_BOX(GTK_FILE_SELECTION(filebrowser)->action_area),
		       bbox, TRUE, TRUE, 0);
      add_selected  = gtk_button_new_with_label(_("Add selected files"));
      gtk_box_pack_start(GTK_BOX(bbox), add_selected, FALSE, FALSE, 0);
      gtk_signal_connect(GTK_OBJECT(add_selected), "clicked",
			 GTK_SIGNAL_FUNC(filebrowser_add_selected_files), filebrowser);
      add_all = gtk_button_new_with_label(_("Add all files in directory"));
      gtk_box_pack_start(GTK_BOX(bbox), add_all, FALSE, FALSE, 0);
      gtk_signal_connect(GTK_OBJECT(add_all), "clicked",
			 GTK_SIGNAL_FUNC(filebrowser_add_all_files), filebrowser);
      gtk_widget_show_all(bbox);

      /*
       * This is ugly.
       * Change the Cancel buttons caption to Close.
       */
	
      label = gtk_label_new(_("Close"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
      gtk_container_remove(GTK_CONTAINER(GTK_FILE_SELECTION(filebrowser)->cancel_button),
			   gtk_container_children(GTK_CONTAINER(GTK_FILE_SELECTION(filebrowser)->cancel_button))->data);
      gtk_container_add(GTK_CONTAINER(GTK_FILE_SELECTION(filebrowser)->cancel_button), label);
      gtk_widget_show(label);

      ptr = g_malloc(sizeof (gint));
      *ptr = !!clear_pl_on_ok * 2;
      gtk_object_set_data_full(GTK_OBJECT(filebrowser), "clear_pl_on_ok", ptr, g_free);

      gtk_widget_show(filebrowser);
    }
}

void filebrowser_add_files(GtkFileSelection * filesel)
{
	GList *sel_list = NULL, *node;
	gchar *text, *text2, *ptr;
	gint *clear_on_ok;

	g_free(Info.sFilename);

	/*
	 * There got to be some clean way to do this too
	 */
	gtk_label_get(GTK_LABEL(GTK_BIN(filesel->history_pulldown)->child), &ptr);
	/* This will give an extra slash if the current dir is the root. */
	Info.sFilename = g_strconcat(ptr, "/", NULL);

	node = GTK_CLIST(filesel->file_list)->selection;
	while (node)
	{
		sel_list = g_list_prepend(sel_list, node->data);
		node = g_list_next(node);
	}
	sel_list = g_list_sort(sel_list, int_compare_func);

	/* if (cfg.open_rev_order) */
/* 		node = g_list_last(sel_list); */
/* 	else */
		node = sel_list;

	clear_on_ok = gtk_object_get_data(GTK_OBJECT(filesel),
					  "clear_pl_on_ok");
	if (clear_on_ok && *clear_on_ok == 2)
	{
		/*
		 * clear the playlist if we're called
		 * from the eject button
		 */
		playlist_clear();
		*clear_on_ok = 1;
	}

	
	if (node)
	{
		/* freeze the playlist to avoid flickering ... */
	  playlistwin_prepare_to_update();
	  while (node)
		{
			gtk_clist_get_text(GTK_CLIST(filesel->file_list), GPOINTER_TO_INT(node->data), 0, &text);
			text2 = g_strconcat(Info.sFilename, text, NULL);
			playlist_add(text2);
			g_free(text2);
			/* if(cfg.open_rev_order) */
/* 				node = g_list_previous(node); */
/* 			else */
				node = g_list_next(node);
		}
	  /* Apply the changes -> thaw the list */
	  playlistwin_update_list();
	}
	else
	{
		/*
		 * No files selected, but the user may have
		 * typed a filename.
		 */
		text = gtk_file_selection_get_filename(filesel);
		if (text[strlen(text) - 1] != '/')
			playlist_add(text);
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel), "");
	}
	g_list_free(sel_list);
	// playlistwin_update_list();
}

void filebrowser_ok(GtkWidget * w, GtkWidget * filesel)
{
	gboolean *clear_on_ok = gtk_object_get_data(GTK_OBJECT(filesel),
						    "clear_pl_on_ok");
	if (util_filebrowser_is_dir(GTK_FILE_SELECTION(filesel)))
		return;
	gtk_widget_hide(filesel);
	filebrowser_add_files(GTK_FILE_SELECTION(filesel));
	if (clear_on_ok && *clear_on_ok > 0)
		playlist_play();
	
	gtk_filebrowser_destroy(NULL, NULL);
}

void filebrowser_add_selected_files(GtkWidget * w, gpointer data)
{
	GtkFileSelection *filesel = GTK_FILE_SELECTION(data);
	
	filebrowser_add_files(filesel);
	gtk_clist_unselect_all(GTK_CLIST(filesel->file_list));

	/*HACK*/
	gtk_entry_set_text(GTK_ENTRY(filesel->selection_entry), "");
}

void filebrowser_add_all_files(GtkWidget * w, gpointer data)
{
	GtkFileSelection *filesel = GTK_FILE_SELECTION(data);

	gtk_clist_freeze(GTK_CLIST(filesel->file_list));
	gtk_clist_select_all(GTK_CLIST(filesel->file_list));
	filebrowser_add_files(filesel);
	/*
	 * We want gtk_clist_undo_selection() but it seems to be buggy
	 * in GTK+ 1.2.6
	 */
	/*  gtk_clist_undo_selection(GTK_CLIST(GTK_FILE_SELECTION(filesel)->file_list)); */
	gtk_clist_unselect_all(GTK_CLIST(filesel->file_list));
	gtk_clist_thaw(GTK_CLIST(filesel->file_list));

	gtk_entry_set_text(GTK_ENTRY(filesel->selection_entry), "");
}

int int_compare_func(gconstpointer a, gconstpointer b)
{
	if (GPOINTER_TO_INT(a) < GPOINTER_TO_INT(b))
		return -1;
	if (GPOINTER_TO_INT(a) > GPOINTER_TO_INT(b))
		return 1;
	else
		return 0;
}

void filebrowser_changed(GtkWidget * w, GtkFileSelection * filesel)
{
 
}

gboolean util_filebrowser_is_dir(GtkFileSelection * filesel)
{
	gchar *text;
	struct stat buf;
	
	text = g_strdup(gtk_file_selection_get_filename(filesel));
	g_strstrip(text);

	if (stat(text, &buf) == 0)
	{
		if (S_ISDIR(buf.st_mode))
		{		/* Selected directory */
			gtk_file_selection_set_filename(filesel, text);
			g_free(text);
			return TRUE;
		}
	}
	g_free(text);
	return FALSE;
}
